SET SQL DIALECT 3;
SET NAMES win1251;

CREATE DATABASE 'st20:e:\database\pm_database.gdb'
USER 'gamer' PASSWORD 'vermut'

DEFAULT CHARACTER SET WIN1251;

/* **************************************************************************** */
CREATE DOMAIN T_INTEGER
 AS INTEGER;
 
CREATE DOMAIN T_UINT64
 AS NUMERIC(18,0);
 
CREATE DOMAIN T_TIMESTAMP
 AS TIMESTAMP;

CREATE DOMAIN T_NAME
 AS VARCHAR(40);

commit;

/* **************************************************************************** */
CREATE GENERATOR GEN_MACHINE_ID;

CREATE TABLE PM_MACHINE
(
 MACHINE_NAME  T_NAME NOT NULL,
 ID_MACHINE    T_INTEGER NOT NULL,
    
 CONSTRAINT PK_PM_MACHINE PRIMARY KEY (ID_MACHINE)
);/*PM_MACHINE*/

/* **************************************************************************** */
SET TERM !! ;

/* Trigger: MACHINE_BI */
CREATE TRIGGER MACHINE_BI FOR PM_MACHINE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID_MACHINE IS NULL) THEN
    NEW.ID_MACHINE = GEN_ID(GEN_MACHINE_ID,1);
END!!

SET TERM ; !!

/* **************************************************************************** */
CREATE GENERATOR GEN_PROCESS_ID;

CREATE TABLE PM_PROCESS
(
 ID_PROCESS        T_INTEGER NOT NULL,
 MACHINE           T_INTEGER NOT NULL,
 NAME              T_NAME NOT NULL,
 PID               T_INTEGER NOT NULL,
 DATE_CREATE       T_TIMESTAMP NOT NULL,
 LAST_UPDATE       T_TIMESTAMP DEFAULT 'NOW' NOT NULL,
 LAST_CHECK        T_TIMESTAMP DEFAULT 'NOW' NOT NULL,
 
 CONSTRAINT UNQ1_PROCESS UNIQUE (NAME, PID, DATE_CREATE, MACHINE),
 CONSTRAINT PK_PM_PROCESS PRIMARY KEY (ID_PROCESS),
 CONSTRAINT FK_PROCESS_1 FOREIGN KEY (MACHINE)
  REFERENCES PM_MACHINE (ID_MACHINE)
  ON DELETE CASCADE
);/*PM_PROCESS*/

/* **************************************************************************** */
SET TERM !! ;

/* Trigger: PROCESS_BI */
CREATE TRIGGER PROCESS_BI FOR PM_PROCESS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID_PROCESS IS NULL) THEN
    NEW.ID_PROCESS = GEN_ID(GEN_PROCESS_ID,1);
END!!

SET TERM ; !!

/* **************************************************************************** */
CREATE GENERATOR GEN_PROCESS_LOG_ID;

CREATE TABLE PM_PROCESS_LOG
(
 PROCESS_LOG    T_INTEGER NOT NULL,
 PROCESS        T_INTEGER,
 LAST_UPDATE    T_TIMESTAMP DEFAULT 'NOW',
 HANDLE_COUNT   T_INTEGER,
 THREAD_COUNT   T_INTEGER,
 READ_COUNT     T_UINT64,
 READ_BYTE      T_UINT64,
 WRITE_COUNT    T_UINT64,
 WRITE_BYTE     T_UINT64,
 OTHER_COUNT    T_UINT64,
 OTHER_BYTE     T_UINT64,
 TIME_CORE      T_UINT64,
 TIME_USER      T_UINT64,

 CONSTRAINT PK_PM_PROCESS_LOG
  PRIMARY KEY (PROCESS_LOG),
  
 CONSTRAINT FK_PROCESS_LOG_1 
  FOREIGN KEY (PROCESS) REFERENCES PM_PROCESS (ID_PROCESS)
  ON DELETE CASCADE
);/*PM_PROCESS_LOG*/

/* **************************************************************************** */
SET TERM !! ;

/* Trigger: PROCESS_LOG_BI */
CREATE TRIGGER PM_PROCESS_LOG_BI FOR PM_PROCESS_LOG
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.PROCESS_LOG IS NULL) THEN
    NEW.PROCESS_LOG = GEN_ID(GEN_PROCESS_LOG_ID,1);
END!!

SET TERM ; !!

/* **************************************************************************** */
SET TERM !! ;


/* Procedure.          .
    -     . */
CREATE PROCEDURE INSERT_NEW_PROCESS (
    ID_MACH INTEGER,
    DATE_CREATE TIMESTAMP,
    PID INTEGER,
    NAME VARCHAR(40),
    ARG_HANDLE_COUNT INTEGER,
    ARG_THREAD_COUNT INTEGER,
    ARG_READ_COUNT NUMERIC(18,0),
    ARG_READ_BYTE NUMERIC(18,0),
    ARG_WRITE_COUNT NUMERIC(18,0),
    ARG_WRITE_BYTE NUMERIC(18,0),
    ARG_OTHER_COUNT NUMERIC(18,0),
    ARG_OTHER_BYTE NUMERIC(18,0),
    ARG_TIME_CORE NUMERIC(18,0),
    ARG_TIME_USER NUMERIC(18,0))
AS
DECLARE VARIABLE ID_PROCESS INTEGER;
DECLARE VARIABLE ID_LOG INTEGER;
DECLARE VARIABLE BD_HANDLE_COUNT INTEGER;
DECLARE VARIABLE BD_THREAD_COUNT INTEGER;
DECLARE VARIABLE BD_READ_COUNT NUMERIC(18,0);
DECLARE VARIABLE BD_READ_BYTE NUMERIC(18,0);
DECLARE VARIABLE BD_WRITE_COUNT NUMERIC(18,0);
DECLARE VARIABLE BD_WRITE_BYTE NUMERIC(18,0);
DECLARE VARIABLE BD_OTHER_COUNT NUMERIC(18,0);
DECLARE VARIABLE BD_OTHER_BYTE NUMERIC(18,0);
DECLARE VARIABLE BD_TIME_CORE NUMERIC(18,0);
DECLARE VARIABLE BD_TIME_USER NUMERIC(18,0);
BEGIN
 SELECT ID_PROCESS
 FROM PM_PROCESS 
 WHERE NAME = :NAME AND
       PID  = :PID  AND
       DATE_CREATE = :DATE_CREATE AND
       MACHINE = :ID_MACH
 INTO :ID_PROCESS;
 

 /*   ,    . */
 IF (ID_PROCESS IS NOT NULL) THEN
 BEGIN
  SELECT MAX(PL2.PROCESS_LOG)
  FROM PM_PROCESS_LOG PL2
  WHERE PL2.PROCESS = :ID_PROCESS
  INTO :ID_LOG;

  SELECT PL.HANDLE_COUNT,
         PL.THREAD_COUNT,
         PL.READ_COUNT,
         PL.READ_BYTE,
         PL.WRITE_COUNT,
         PL.WRITE_BYTE,
         PL.OTHER_COUNT,
         PL.OTHER_BYTE,
         PL.TIME_CORE,
         PL.TIME_USER
         FROM PM_PROCESS_LOG PL
  WHERE PL.PROCESS_LOG =:ID_LOG
  INTO   :BD_HANDLE_COUNT, 
         :BD_THREAD_COUNT,
         :BD_READ_COUNT,
         :BD_READ_BYTE,
         :BD_WRITE_COUNT,
         :BD_WRITE_BYTE,
         :BD_OTHER_COUNT,
         :BD_OTHER_BYTE,
         :BD_TIME_CORE,
         :BD_TIME_USER;

  /*   . */
  IF (BD_HANDLE_COUNT <> ARG_HANDLE_COUNT OR
      BD_THREAD_COUNT <> ARG_THREAD_COUNT OR
      BD_READ_COUNT   <> ARG_READ_COUNT   OR
      BD_READ_BYTE    <> ARG_READ_BYTE    OR
      BD_WRITE_COUNT  <> ARG_WRITE_COUNT  OR
      BD_WRITE_BYTE   <> ARG_WRITE_BYTE   OR
      BD_OTHER_COUNT  <> ARG_OTHER_COUNT  OR
      BD_OTHER_BYTE   <> ARG_OTHER_BYTE   OR
      BD_TIME_CORE    <> ARG_TIME_CORE    OR
      BD_TIME_USER    <> ARG_TIME_USER       ) THEN
   BEGIN

    /*       pm_process_log*/
    INSERT INTO pm_process_log (PROCESS,
                                HANDLE_COUNT,
                                THREAD_COUNT,
                                READ_COUNT,
                                READ_BYTE,
                                WRITE_COUNT,
                                WRITE_BYTE,
                                OTHER_COUNT,
                                OTHER_BYTE,
                                TIME_CORE,
                                TIME_USER)
    VALUES(
     :ID_PROCESS,
     :ARG_HANDLE_COUNT,
     :ARG_THREAD_COUNT,
     :ARG_READ_COUNT,
     :ARG_READ_BYTE,
     :ARG_WRITE_COUNT,
     :ARG_WRITE_BYTE,
     :ARG_OTHER_COUNT,
     :ARG_OTHER_BYTE,
     :ARG_TIME_CORE,
     :ARG_TIME_USER
    );

    UPDATE PM_PROCESS SET LAST_UPDATE = 'NOW'
     WHERE ID_PROCESS = :ID_PROCESS;

  END

  /*      .*/
  UPDATE PM_PROCESS SET LAST_CHECK = 'NOW'
  WHERE ID_PROCESS = :ID_PROCESS;
 END
 ELSE
 BEGIN
  /*    ,     PM_PROCESS*/

  INSERT INTO PM_PROCESS (
                          NAME,
                          PID,
                          DATE_CREATE,
                          MACHINE
                         )
  VALUES(
         :NAME,
         :PID,
         :DATE_CREATE,
         :ID_MACH
        );

  /*      . */
  SELECT ID_PROCESS
  FROM PM_PROCESS
  WHERE NAME = :NAME AND
        PID  = :PID  AND
        DATE_CREATE = :DATE_CREATE AND
        MACHINE = :ID_MACH
  INTO :ID_PROCESS;

  /*     PM_PROCESS_LOG */
  INSERT INTO PM_PROCESS_LOG(
                             PROCESS,
                             HANDLE_COUNT,
                             THREAD_COUNT,
                             READ_COUNT,
                             READ_BYTE,
                             WRITE_COUNT,
                             WRITE_BYTE,
                             OTHER_COUNT,
                             OTHER_BYTE,
                             TIME_CORE,
                             TIME_USER
                            )
  VALUES (
          :ID_PROCESS,
          :ARG_HANDLE_COUNT,
          :ARG_THREAD_COUNT,
          :ARG_READ_COUNT,
          :ARG_READ_BYTE,
          :ARG_WRITE_COUNT,
          :ARG_WRITE_BYTE,
          :ARG_OTHER_COUNT,
          :ARG_OTHER_BYTE,
          :ARG_TIME_CORE,
          :ARG_TIME_USER
         );

 END

 SUSPEND;
END!!

SET TERM ; !!

/* **************************************************************************** */
COMMIT;
/* */