'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'     XML-
'                                                 . 24.08.2005

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
option explicit

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'

const c_xml_wrt_si_is_started               =1
const c_xml_wrt_si_has_child                =2

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_xml_writer_stack_item

class t_xml_writer_stack_item
 dim tag_name
 dim flags
end class 't_xml_writer_stack_item
 
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_xml_file_writer_stack

class t_xml_file_writer_stack
 private c_err_source
 
 private m_file
 private m_indent

 private m_stack ' 
 private m_end_tag_str


 'constructor ------------------------------------------------------------
 private sub class_initialize()
  c_err_source="t_xml_file_writer_stack"
  
  m_indent    =false
  
  set m_file  =nothing
  set m_stack =su_create_object("LCPI.DBS.Utility.Vector")
 end sub 'class_initialize

 '------------------------------------------------------------------------
 private sub class_terminate()
  if(not (m_file is nothing))then
   call flush()
  end if
 end sub 'class_terminate

 'properties -------------------------------------------------------------
 public property set file(x)
  set m_file=x
  
  call m_stack.Clear()
 end property

 '------------------------------------------------------------------------
 public property get file()
  set file=m_file
 end property 'file

 '------------------------------------------------------------------------
 public property let indent(x)
  m_indent=x
 end property 'let indent

 '------------------------------------------------------------------------
 public property get indent()
  indent=m_indent
 end property 'get indent

 '------------------------------------------------------------------------
 'attrs - Scripting.Dictionary
 
 public sub push_tag(tag_name,attrs)
  '      
  if(not m_stack.IsEmpty)then
   call out_end_tag()
   
   m_stack.Back.flags=m_stack.Back.flags or c_xml_wrt_si_has_child
  end if

  dim x
  set x=new t_xml_writer_stack_item
  
  x.tag_name     =tag_name
  x.flags        =0

  call m_stack.PushBack(x)
  
  call build_end_tag_str(tag_name,attrs)
 end sub 'push_tag

 '------------------------------------------------------------------------
 public sub push_closed_tag(tag_name,attrs)
  '      
  if(not m_stack.IsEmpty)then
   call out_end_tag()

   m_stack.Back.flags=m_stack.Back.flags or c_xml_wrt_si_has_child
  end if

  call out_indent(m_stack.Count>0,m_stack.Count)

  call build_end_tag_str(tag_name,attrs)

  call m_file.Write("<"&m_end_tag_str&"/>")
 end sub 'push_closed_tag

 '------------------------------------------------------------------------
 public sub push_closed_tag_ex(tag_name,attrs,characters)
  '      
  if(not m_stack.IsEmpty)then
   call out_end_tag()

   m_stack.Back.flags=m_stack.Back.flags or c_xml_wrt_si_has_child
  end if

  call out_indent(m_stack.Count>0,m_stack.Count)

  call build_end_tag_str(tag_name,attrs)

  if(characters="")then
   call m_file.Write("<"&m_end_tag_str&"/>")
  else
   call m_file.Write("<"&m_end_tag_str&">")
   call m_file.Write(characters)
   call m_file.Write("</"&tag_name&">")
  end if
 end sub 'push_closed_tag_ex

 '------------------------------------------------------------------------
 public sub push_comment(text)
  '      
  if(not m_stack.IsEmpty)then
   call out_end_tag()

   m_stack.Back.flags=m_stack.Back.flags or c_xml_wrt_si_has_child
  end if

  call out_indent(m_stack.Count>0,m_stack.Count)

  call m_file.WriteLine("<!-- "&text&" -->")
 end sub 'push_comment

 '------------------------------------------------------------------------
 private sub build_end_tag_str(tag_name,attrs)
  m_end_tag_str  =tag_name

  if(not (attrs is nothing))then
   dim a

   for each a in attrs.Keys
    m_end_tag_str=m_end_tag_str&" "&a&"="""&cstr(attrs(a))&""""
   next 'a
  end if
 end sub 'build_end_tag_str

 '------------------------------------------------------------------------
 public sub pop_tag()
  if(m_stack.IsEmpty())then
   call err.raise(-1,c_err_source,"Stack is empty")
  end if
  
  dim back
  set back=m_stack.Back

  if((back.flags and c_xml_wrt_si_is_started)<>0)then
   ' </tag_name>
   call out_indent(true,m_stack.Count-1)
   
   call m_file.Write("</"&back.tag_name&">")
  else
   ' <end_tag_str/>
   call out_indent(m_stack.Count>1,m_stack.Count-1)

   call m_file.Write("<"&m_end_tag_str&"/>")
  end if
  
  call m_stack.PopBack()
 end sub 'pop_tag
 
 '------------------------------------------------------------------------
 public sub characters(str)
  if(not m_stack.IsEmpty)then
   call out_end_tag()
  end if

  'out text
  call m_File.Write(str)
 end sub 'characters
 
 '------------------------------------------------------------------------
 public sub flush()
  while(not m_stack.IsEmpty)
   call pop_tag()
  wend
 end sub 'flush

 '------------------------------------------------------------------------
 private sub out_end_tag()
  dim back
  set back=m_stack.Back

  if((back.flags and c_xml_wrt_si_is_started)<>0)then
   exit sub
  end if

  back.flags=back.flags or c_xml_wrt_si_is_started

  call out_indent(m_stack.Count>1,m_stack.Count-1)

  '<end_tag_name>
  call m_file.Write("<"&m_end_tag_str&">")
 end sub 'out_end_tag
 
 '------------------------------------------------------------------------
 private sub out_indent(new_line,n)
  if(m_indent<>true)then
   exit sub
  end if
 
  if(new_line)then
   call m_file.WriteLine("")
  end if
  
  dim i
  for i=1 to n
   call m_file.write(" ")
  next 'i
 end sub 'out_indent
end class 't_xml_file_writer_stack

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

