'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'   
'                                          . 27  2004 .

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
option explicit

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_master_task_descr

class t_master_task_descr
 private  m_task
 private  m_success '   
 
 'constructor ------------------------------------------------------------
 private sub class_initialize()
  set m_task    =nothing
  m_success     =true
 end sub
 
 'properties -------------------------------------------------------------
 public property set task(x)
  set m_task=x
 end property
 
 public property let success(x)
  m_success=x
 end property
 
 '------------------------------------------------------------------------
 public property get task()
  set task=m_task
 end property
 
 public property get success()
  success=m_success
 end property
end class 't_master_task_descr

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_task_base_code

class t_task_base_code
 private c_err_source

 private m_prog
 private m_master_tasks         'vector of t_master_task_descr
 private m_log
 private m_error_source
 
 'constructor ------------------------------------------------------------
 private sub class_initialize()
  c_err_source="t_task_base_code"
 
  set m_prog=nothing
  set m_master_tasks=su_create_object("LCPI.DBS.Utility.Vector")
  set m_log=nothing
 end sub 'class_initialize
 
 'properties -------------------------------------------------------------
 public property set prog(x)
  set m_prog=x
 end property
 
 '------------------------------------------------------------------------
 public property get prog()
  set prog=m_prog
 end property
 
 '------------------------------------------------------------------------
 public property set log(x)
  set m_log=x
 end property

 '------------------------------------------------------------------------
 public property get log()
  set log=m_log
 end property
 
 '------------------------------------------------------------------------
 public property let error_source(x)
  m_error_source=x
 end property
 
 '------------------------------------------------------------------------
 public property get error_source()
  error_source=m_error_source
 end property
 
 '     ---------------------------------
 public sub depended_on_task(task,success)
  if(task is nothing)then
   call err.raise(-1,c_err_source,"depended_on_task: NULL pointer on task")
  end if
 
  dim td
  set td=new t_master_task_descr
 
  set td.task =task
  td.success  =success
 
  call m_master_tasks.PushBack(td)
 end sub 'depended_on_task
 
 '   --------------------------------------------
 public sub clear_master_tasks()
  call m_master_tasks.clear()
 end sub
 
 '    -----------------------------------
 public function can_start()
  can_start=false

  dim td

  '   ,      
  for each td in m_master_tasks
   if(not td.task.is_finished())then
    exit function
   end if

   if(td.success and td.task.get_exit_code()<>0)then
    exit function
   end if
  next 'task

  can_start=true
 end function 'can_start

 '    --------------------------------------
 public function is_active()
  is_active=false

  if(not(m_prog is nothing))then
   is_active=m_prog.is_active()
  end if
 end function 'is_active

 '------------------------------------------------------------------------
 public function is_finished()
  is_finished=false

  if(m_prog is nothing)then
   exit function '  
  end if

  if(m_prog.is_active())then
   exit function '  
  end if

  is_finished=true
 end function 'is_finished

 '------------------------------------------------------------------------
 public function get_exit_code()
  if(m_prog is nothing)then
   get_exit_code=-1
  else
   get_exit_code=m_prog.get_exit_code()
  end if
 end function 'get_exit_code

 'trace methods ----------------------------------------------------------
 public sub log_msg(msg)
  if(not (m_log is nothing))then
   call m_log.send_msg(msg)
  end if
 end sub 'log_msg

 '------------------------------------------------------------------------
 public sub log_error(err_code,err_descr)
  if(not(m_log is nothing))then
   call m_log.send_error(err_code,m_error_source,err_descr)
  end if
 end sub 'log_error

 '------------------------------------------------------------------------
 public sub log_warning(err_code,err_descr)
  if(not(m_log is nothing))then
   call m_log.send_warning(err_code,m_error_source,err_descr)
  end if
 end sub 'log_warning
 '------------------------------------------------------------------------
 public sub log_success(err_descr)
  if(not(m_log is nothing))then
   call m_log.send_success(m_error_source,err_descr)
  end if
 end sub 'log_success

end class 't_task_base_code

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

