'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'     
'     
'                                                . 16.08.2004

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
option explicit

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_task_for_archive_installed_rpl_packs

class t_task_for_archive_installed_rpl_packs
 private m_base
 private m_fso

 private m_databases '        

 private m_gbak_exe
 private m_gdb_pack_files_dir  '     
 private m_gbk_pack_files_dir  '     
 private m_history_dir         'class t_archive_cat

 private m_rpr_tasks           '     .

 Private m_smtp_server         ' SMTP      
 Private m_all_send_mail       '          
 Private m_terrain_send_mail   '       ,     

 private m_exit_code
 private m_is_finished

 'constructor ------------------------------------------------------------
 private sub class_initialize()
  set m_base      =new t_task_base_code
  set m_fso       =su_create_object("Scripting.FileSystemObject")
  set m_databases =su_create_object("LCPI.DBS.Utility.Vector")
  set m_rpr_tasks =su_create_object("LCPI.DBS.Utility.Vector")

  set m_history_dir =nothing

  m_smtp_server           = Null
  Set m_all_send_mail     = Nothing
  Set m_terrain_send_mail = Nothing

  m_base.error_source=typename(me)

  m_exit_code    =-1
  m_is_finished  =false
 end sub 'class_initialize

  '-----------------------------------------------------------------------------
  '                              Private 
  '-----------------------------------------------------------------------------

  Public Property Get title()
    title = "TASK FOR ARCHIVE INSTALLED RPL PACKS"
  End Property  ' title

  '-----------------------------------------------------------------------------
  ' SMTP      
  Public Property Let smtp_server(value)
    m_smtp_server = value
  End Property  ' Let smtp_server

  '-----------------------------------------------------------------------------
  '          
  Public Property Set all_send_mail(value)
    Set m_all_send_mail = value
  End Property  ' Set all_send_mail

  '-----------------------------------------------------------------------------
  '       ,     
  Public Property Set terrain_send_mail(value)
    Set m_terrain_send_mail = value
  End Property  ' Set terrain_send_mail


 ' ----------------------------------------------------------
 public sub init(log,                _
                 gbak_exe,           _
                 gdb_pack_files_dir, _
                 gbk_pack_files_dir, _
                 history_dir)

  set m_base.log        = log
  m_gbak_exe            = gbak_exe
  m_gdb_pack_files_dir  = gdb_pack_files_dir
  m_gbk_pack_files_dir  = gbk_pack_files_dir

  set m_history_dir     = history_dir

  m_exit_code    =-1
  m_is_finished  =false
 end sub 'init

 '------------------------------------------------------------------------
 '         
 '      

 public sub reg_database(db_location)
  call m_databases.PushBack(db_location)
 end sub 'reg_database

 '------------------------------------------------------------------------
 public sub depended_on_task(task)
  call m_base.depended_on_task(task,true)

  '    
  if(typename(task)="t_rpl_for_restore_rpl_packs")then
   call m_rpr_tasks.PushBack(task)
  end if
 end sub 'depended_on_task

 '    -----------------------------------
 public function can_start()
  can_start=m_base.can_start()
 end function 'can_start

 '    --------------------------------------
 public function is_active()
  is_active=false
 end function 'is_active

 '------------------------------------------------------------------------
 public function get_exit_code()
  get_exit_code=m_exit_code
 end function 'get_exit_code

 '------------------------------------------------------------------------
 public function is_finished()
  is_finished=m_is_finished
 end function 'is_finished

 '  ---------------------------------------------------------
 public function run(mng)
  run=false

  '---------------------
  dim gdb_files
  dim file
  dim i,n

  dim err_code
  dim err_descr

  '---------------------
  call m_base.log_msg("Scan rpl_pack databases ...")

  set gdb_files=su_create_object("LCPI.DBS.Utility.Vector")

  if(not su_scan_files(m_base.log,gdb_files,m_gdb_pack_files_dir,"gdb"))then
   exit function
  end if

  '  
  for i=0 to gdb_files.count-1
   set file=new t_rpl_pack_file

   set file.gdb_file=gdb_files(i)

   gdb_files(i)=file
  next 'i

  '---------------------  GDB    
  dim gdb_files_with_failed_rpl_packs
  set gdb_files_with_failed_rpl_packs=su_create_object("Scripting.Dictionary")

  for i=0 to m_rpr_tasks.Count-1
   for each n in m_rpr_tasks(i).fail_rpl_packs
    gdb_files_with_failed_rpl_packs(n.gdb_file.name)=1
   next 'n
  next 'i

  '---------------------
  call m_base.log_msg("Scan rpl_packs ...")

  for i=0 to gdb_files.count-1
   set file=gdb_files(i)

   if(gdb_files_with_failed_rpl_packs.Exists(file.gdb_file.name))then
    call m_base.log_warning(1,"GDB FILE ["&file.gdb_file.Path&"] has failed rpl pack")

    set gdb_files(i)=nothing
   else
    on error resume next

    call load_rpl_pack_descrs("localhost",file.gdb_file,file.rpl_packs)

    err_code  =err.number
    err_descr =err.description

    call err.clear()
    on error goto 0

    if(err_code<>0)then
     '      
     call m_base.log_error(err_code,"SCAN GDB FILE ["&file.gdb_file.Path&"]"&vbCrLf&err_descr)

     '    
     set gdb_files(i)=nothing
    else
     call m_base.log_msg(file.gdb_file.Name&": "&file.rpl_packs.count&" packs.")
    end if
   end if 'else
  next 'file

  '---------------------
  dim db_location

  for each db_location in m_databases
   call scan_installed_rpl_packs(db_location,gdb_files)
  next 'descr

  '---------------------
  if(not m_fso.FolderExists(m_gbk_pack_files_dir))then
   call m_base.log_error(-1,"BACKUP FOLDER ["&m_gbk_pack_files_dir&"] NOT FOUND.")

   exit function
  end if

  '---------------------
  if(not m_fso.FolderExists(m_history_dir.path))then
   call m_base.log_error(-1,"HISTORY FOLDER ["&m_history_dir.path&"] NOT FOUND.")

   exit function
  end if

  '---------------------
  '    ,    
  '    

  for each file in gdb_files
   if(not(file is nothing))then
    call archive_rpl_pack_file(mng, file)
   end if
  next 'file

  m_exit_code=0
  m_is_finished=true

  run=true
 end function 'run

 '------------------------------------------------------------------------
 public sub deactivate(mng)

 end sub 'deactivate

 '------------------------------------------------------------------------
 '    [db_location]
 '        nothing
 private sub scan_installed_rpl_packs(byval db_location,gdb_files)
  dim cn,msg

  m_base.log_msg("Scan installed packs in ["&db_location&"]...")

  set cn=su_create_object("ADODB.Connection")

  cn.Provider="LCPI.IBProvider"

  call cn.Open("location="&db_location&";ctype=win1251;ole db services=0;", _
               "gamer","vermut")

  cn.IsolationLevel=65536

  call cn.BeginTrans()

  '--------------------------------------------------------
  '    

  dim current_terrain_descr
  set current_terrain_descr=rpl_get_current_terrain_descr(cn)

  dim current_terrain_global_id

  if(current_terrain_descr is nothing)then
   current_terrain_global_id=null
  else
   current_terrain_global_id=current_terrain_descr.global_id
  end if

  '--------------------------------------------------------
  dim cmd
  set cmd=su_create_object("ADODB.Command")

  cmd.ActiveConnection=cn

  cmd.CommandText= _
   "select p.rec_id"&vbCrLf& _
   "from rpl_restored_pack p join rpl_terrain t on p.terrain_id=t.terrain_id"&vbCrLf&_
   "where t.terrain_global_id=? and p.pack_id=?"

  dim i,pack

  for i=0 to gdb_files.count-1
   if(not (gdb_files(i) is nothing))then
    for each pack in gdb_files(i).rpl_packs

     '     
     if(not su_equal_db_value(current_terrain_global_id,pack.terrain_global_id))then
      cmd(0)=pack.terrain_global_id
      cmd(1)=pack.pack_id

      if(cmd.execute.eof)then
       '   -      
       msg=pack.terrain_name&vbCrLf& _
           "PACK ["&cstr(pack.terrain_global_id)&":"&cstr(pack.pack_id)&"].["&pack.gdb_file.name&"] "& _
           "is not restored"

       call m_base.log_warning(1,msg)

       gdb_files(i)=nothing

       exit for
      end if '  

     end if '      
    next 'pack
   end if 'gdb_file(i) is not nothing
  next 'i

  call cn.CommitTrans()
 end sub 'scan_installed_rpl_packs

  '------------------------------------------------------------------------
  Private Sub archive_rpl_pack_file(mng, file)
    Dim gdb_file : Set gdb_file = file.gdb_file

    call m_base.log_msg("Archive rpl_pack file: "& gdb_file.Name)

    '1     
    '2  () 
    '3        
    '4    

    dim backup_task

    '    
    dim gbk_location

    gbk_location=m_fso.BuildPath(m_gbk_pack_files_dir,m_fso.GetBaseName(gdb_file.Name)&".gbk")

    '1
    if(m_fso.FileExists(gbk_location))then
     backup_task=gbk_location
    else
     set backup_task=new t_task_for_backup_db

     dim db_descr
     set db_descr=new t_db_descr

     db_descr.host      ="localhost"
     db_descr.location  =gdb_file.Path
     db_descr.user_name ="gamer"
     db_descr.password  ="vermut"

     call backup_task.init(m_base.log,m_gbak_exe,db_descr,gbk_location)

     call mng.add_task(backup_task)
    end if

    '2
    dim zip_task
    set zip_task=new t_task_for_zip_backup_files

    call zip_task.init(m_base.log,backup_task,m_history_dir)

    zip_task.add_ts_suffix=true

    call mng.add_task(zip_task)

    '3.1
    dim remove_gbk_task
    set remove_gbk_task=new t_task_for_remove_backup_files

    call remove_gbk_task.init(m_base.log,backup_task)

    '    __ 
    call remove_gbk_task.depended_on_task(zip_task,true)

    call mng.add_task(remove_gbk_task)

    '3.2
    '          
    '   ,       
    '     

    dim remove_gdb_task
    set remove_gdb_task=new t_task_for_remove_gdb_file
    set remove_gdb_task.log=m_base.log

    call remove_gdb_task.init(gdb_file.Path)

    '   
    if(IsObject(backup_task))then
     call remove_gdb_task.depended_on_task(backup_task)
    end if

    call mng.add_task(remove_gdb_task)

    '4
    Dim pack, cfg_file, cfg_files

    '    ,       
    If (su_is_not_empty_dictionary(m_terrain_send_mail)) Then
      '      ,      
      '      ,   
      For Each pack In file.rpl_packs
        If (m_terrain_send_mail.Exists(pack.terrain_global_id)) Then
          cfg_files = m_terrain_send_mail(pack.terrain_global_id).Keys

          For Each cfg_file In cfg_files
            If (su_is_not_empty_dictionary(m_all_send_mail)) Then
              If (Not m_all_send_mail.Exists(cfg_file)) Then
                Call add_task_send_mail_archive(mng, cfg_file, zip_task)
              End If
            Else
              Call add_task_send_mail_archive(mng, cfg_file, zip_task)
            End If
          Next

          Exit For
        End If
      Next  ' pack
    End If

    '        ,     
    If (su_is_not_empty_dictionary(m_all_send_mail)) Then
      cfg_files = m_all_send_mail.Keys

      For Each cfg_file In cfg_files
        Call add_task_send_mail_archive(mng, cfg_file, zip_task)
      Next
    End If

  End Sub  ' archive_rpl_pack_file

  '-----------------------------------------------------------------------------
  '        file
  Private Sub add_task_send_mail_archive(mng, cfg_file, zip_task)
    Dim send_archive_task : Set send_archive_task = New t_task_for_send_archive_by_mail

    Call send_archive_task.init(m_base.log, m_smtp_server, cfg_file, " ", zip_task)

    '     ,    
    Call send_archive_task.depended_on_task(zip_task)

    '    
    Call mng.add_task(send_archive_task)
  End Sub  ' add_task_send_mail_archive

  '-----------------------------------------------------------------------------
End Class  ' t_task_for_archive_installed_rpl_packs

'*******************************************************************************