'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'      
'                                       . 3  2004 .

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
option explicit

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_task_for_finish_rpl_backup

class t_task_for_finish_rpl_backup
 private c_err_source
 
 private m_base
 private m_fso
 private m_pack_descr  't_db_descr
 private m_target_dir
 
 private m_exit_code
 private m_is_finished

 'constructor ------------------------------------------------------------
 private sub class_initialize()
  c_err_source="t_task_for_finish_rpl_backup"
  
  set m_base=new t_task_base_code
  set m_fso =su_create_object("Scripting.FileSystemObject")
  
  set m_pack_descr=nothing
  
  m_base.error_source=c_err_source

  m_exit_code   =-1
  m_is_finished =false
 end sub 'class_initialize
 
 'properties -------------------------------------------------------------
 public property set log(x)
  set m_base.log=x
 end property

 '------------------------------------------------------------------------
 property get title()
  title=c_err_source
 end property

 '------------------------------------------------------------------------
 public property get task_for_finish_packs_restore()
  set task_for_finish_packs_restore=m_task_for_finish_packs_restore
 end property

 ' ----------------------------------------------------------
 public sub init(pack_descr,target_dir)
  set m_pack_descr=pack_descr
 
  m_target_dir=target_dir
 
  m_exit_code   =-1
  m_is_finished =false
 end sub 'init

 '------------------------------------------------------------------------
 public sub depended_on_task(task,success)
  call m_base.depended_on_task(task,success)
 end sub 'depended_on_task

 '    -----------------------------------
 public function can_start()
  can_start=m_base.can_start()
 end function 'can_start

 '    --------------------------------------
 public function is_active()
  is_active=false '     
 end function 'is_active

 '------------------------------------------------------------------------
 public function get_exit_code()
  get_exit_code=m_exit_code
 end function 'get_exit_code

 '------------------------------------------------------------------------
 public function is_finished()
  is_finished=m_is_finished
 end function 'is_finished

 '------------------------------------------------------------------------
 public function run(mng)
  run=false
  
  dim msg
  
  '1         (disconnect)
  '2       ( )
  '3.1   
  '3.2      
  '     -     
  '     - 
  '     -   GDB
  
  '0 --------------------------------------
  dim source_file_dir '   
  source_file_dir=m_fso.GetParentFolderName(m_pack_descr.location)

  dim source_file_ext
  source_file_ext=m_fso.GetExtensionName(m_pack_descr.location)

  dim file_name
  file_name=m_fso.GetFileName(m_pack_descr.location)

  dim new_file_name
  new_file_name=m_fso.GetBaseName(file_name)&"."&source_file_ext&"tmp"

  '---------------------
  if(not m_fso.FileExists(m_pack_descr.location))then
   msg="DATABASE FILE ["&m_pack_descr.location&"] NOT FOUND"
   
   call err.raise(-1,c_err_source,msg)
  end if
  
  if(not m_fso.FolderExists(m_target_dir))then
   msg="TARGET FOLDER ["&m_target_dir&"] NOT FOUND"
   
   call err.raise(-1,c_err_source,msg)
  end if
  
  '      
  if(m_fso.FileExists(m_fso.BuildPath(m_target_dir,file_name)))then
   msg="TARGET FOLDER ["&m_target_dir&"] ALREADY HAS ["&file_name&"]"
   
   call err.raise(-1,c_err_source,msg)
  end if
  
  if(m_fso.FileExists(m_fso.BuildPath(m_target_dir,new_file_name)))then
   msg="TARGET FOLDER ["&m_target_dir&"] ALREADY HAS ["&new_file_name&"]"
   
   call err.raise(-1,c_err_source,msg)
  end if
  
  '1 --------------------------------------
  dim has_pack
  has_pack=rpl_db_has_pack()
  
  '2 --------------------------------------
  call rename_file(m_pack_descr.location,new_file_name) 'throw

  if(not has_pack)then
   call delete_file(m_fso.BuildPath(source_file_dir,new_file_name)) 'throw
  else
   call move_file(m_fso.BuildPath(source_file_dir,new_file_name), _
                  m_fso.BuildPath(m_target_dir,new_file_name)) 'throw
                  
   call rename_file(m_fso.BuildPath(m_target_dir,new_file_name), _
                    file_name) 'throw
  end if
  
  m_is_finished=true
  m_exit_code=0
  
  run=true
 end function 'run
 
 '------------------------------------------------------------------------
 public sub deactivate(mng)
 end sub
 
 '------------------------------------------------------------------------
 private sub rename_file(byval file_location,byval new_file_name) 'throw
  dim msg
  dim err_code
  dim err_descr
  
  on error resume next
  
  m_fso.GetFile(file_location).Name=new_file_name
  
  err_code  =err.number
  err_descr =err.description
  
  call err.Clear()
  
  on error goto 0
  
  if(err_code<>0)then
   msg="FAIL RENAME ["&file_location&"] TO ["&new_file_name&"]"&vbCrLf&_
        err_descr
   
   call err.raise(err_code,c_err_source,msg)
  else
   call m_base.log_msg("RENAME FILE ["&file_location&"] TO ["&new_file_name&"]")
  end if
 end sub 'rename_file
 
 '------------------------------------------------------------------------
 private sub delete_file(file_location)
  dim msg
  dim err_code
  dim err_descr

  on error resume next

  call m_fso.DeleteFile(file_location,true)

  err_code  =err.number
  err_descr =err.description

  call err.Clear()

  on error goto 0

  if(err_code<>0)then
   msg="FAIL DELETE FILE ["&file_location&"]"&vbCrLf&_
        err_descr

   call err.raise(err_code,c_err_source,msg)
  else
   call m_base.log_msg("DELETE FILE ["&file_location&"]")
  end if
 end sub 'delete_file
 
 '------------------------------------------------------------------------
 private sub move_file(source_file_location,target_file_location)
  dim msg
  dim err_code
  dim err_descr

  on error resume next

  call m_fso.MoveFile(source_file_location,target_file_location)

  err_code  =err.number
  err_descr =err.description

  call err.Clear()

  on error goto 0

  if(err_code<>0)then
   msg="FAIL MOVE FILE ["&source_file_location&"] TO ["&target_file_location&"]"&vbCrLf&_
        err_descr

   call err.raise(err_code,c_err_source,msg)
  else
   call m_base.log_msg("MOVE FILE ["&source_file_location&"] TO ["&target_file_location&"]")
  end if
 end sub 'move_file
 
 '------------------------------------------------------------------------
 private function rpl_db_has_pack()
  dim cn
  set cn=su_create_object("ADODB.Connection")
  cn.Provider="LCPI.IBProvider"

  call cn.Open("data source="&m_pack_descr.data_source&";"& _
               "ctype=win1251;ole db services=0", _
               m_pack_descr.user_name, _
               m_pack_descr.password)

  cn.IsolationLevel=65536

  call cn.BeginTrans()

  dim cmd
  set cmd=su_create_object("ADODB.Command")
  cmd.activeconnection=cn

  cmd.commandtext="select * from rpl_pack"

  rpl_db_has_pack=not cmd.execute.eof

  call cn.CommitTrans()
 end function 'rpl_db_has_pack
end class 't_task_for_finish_rpl_backup

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

