'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'    GDB     
'   
'                                         . 2  2004 .

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
option explicit

'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'class t_task_for_scan_rpl_packs

class t_task_for_scan_rpl_packs
 private m_base
 private m_gdb_files ' GDB   
 private m_rpl_packs '  
 
 private m_rpl_gdb_files_dir
 
 private m_exit_code
 private m_is_finished
 
 'constructor ------------------------------------------------------------
 private sub class_initialize()
  set m_base     =new t_task_base_code
  set m_gdb_files=su_create_object("LCPI.DBS.Utility.Vector")
  set m_rpl_packs=su_create_object("LCPI.DBS.Utility.Vector")
  
  m_base.error_source=typename(me)
  
  m_exit_code  =-1
  m_is_finished=false
 end sub 'class_initialize
 
 'properties -------------------------------------------------------------
 public property get title()
  title="SCAN RPL PACKS"
 end property
 
 '------------------------------------------------------------------------
 public property get gdb_files()
  set gdb_files=m_gdb_files
 end property
 
 '------------------------------------------------------------------------
 public property get rpl_packs()
  set rpl_packs=m_rpl_packs
 end property
 
 ' ----------------------------------------------------------
 'rpl_gdb_files_dir -   GDB   
 public sub init(log, rpl_gdb_files_dir)
  set m_base.log=log

  call m_gdb_files.clear()
  call m_rpl_packs.clear()
  
  m_rpl_gdb_files_dir=rpl_gdb_files_dir

  m_exit_code  =-1
  m_is_finished=false
 end sub 'init

 '------------------------------------------------------------------------
 public sub depended_on_task(task)
  call m_base.depended_on_task(task,true)
 end sub 'depended_on_task

 '    -----------------------------------
 public function can_start()
  can_start=m_base.can_start()
 end function 'can_start

 '    --------------------------------------
 public function is_active()
  is_active=false
 end function 'is_active

 '------------------------------------------------------------------------
 public function get_exit_code()
  get_exit_code=m_exit_code
 end function 'get_exit_code

 '------------------------------------------------------------------------
 public function is_finished()
  is_finished=m_is_finished
 end function 'is_finished

 '------------------------------------------------------------------------
 public function run(mng)
  run=false

  if(not su_scan_files(m_base.log,m_gdb_files,m_rpl_gdb_files_dir,"gdb"))then
   exit function
  end if

  call m_base.log_msg("")
  call m_base.log_msg("[BUILD RPL PACK LIST]")

  dim file
  dim err_code
  dim err_descr
  dim n
  
  for each file in m_gdb_files
   on error resume next
   n=load_rpl_pack_descrs("localhost",file,m_rpl_packs)
  
   err_code=err.number
   err_descr=err.description
  
   call err.clear()
   on error goto 0

   if(err_code<>0)then
    call m_base.log_error(err_code,"SCAN GDB FILE ["&file.Path&"]"&vbCrLf&err_descr)
    exit function
   end if
   
   call m_base.log_msg(file.Name&": "&n&" packs.")
  next 'file

  call m_base.log_msg("[SORT RPL PACK LIST]")
  call sort_buble(m_rpl_packs,new t_rpl_pack_descr_compare_for_restore)
  
  if(not test_dublicates_packs())then
   exit function
  end if
  
  if(not test_sequential_packs())then
   exit function
  end if
  
  '   
  m_is_finished=true
  m_exit_code=0

  run=true
 end function 'run
 
 '------------------------------------------------------------------------
 public sub deactivate(mng)
 end sub 'deactivate

 '------------------------------------------------------------------------
 '     
 
 'return: false - m_rpl_packs   
 
 private function test_dublicates_packs()
  test_dublicates_packs=false

  call m_base.log_msg("[TEST RPL PACK DUBLICATES]")

  dim res
  res=0
  
  dim ip1,p1
  dim ip2,p2
  
  dim msg
  
  'dim bad_packs
  'set bad_packs=su_create_object("LCPI.DBS.Utility.Vector")
  
  for ip1=0 to m_rpl_packs.count-1
   set p1=m_rpl_packs(ip1)
   
   for ip2=ip1+1 to m_rpl_packs.count-1
    set p2=m_rpl_packs(ip2)
    
    if(p1.terrain_global_id=p2.terrain_global_id and p1.pack_id=p2.pack_id)then
     '   
     res=res+1
     
     msg="   [Terrain:"&cstr(p1.terrain_name)&" PackID:"&cstr(p1.pack_id)&"]"&vbCrLf&_
         "GDB1:"&p1.gdb_file.path&vbCrLf&_
         "GDB2:"&p2.gdb_file.path
         
     call m_base.log_error(-1,msg)
     
     '  
     'call bad_packs.PushBack(p1)
    end if
   next 'ip2
  next 'ip1
  
  test_dublicates_packs=(res=0)
 end function 'test_dublicates_packs
 
 '------------------------------------------------------------------------
 '    
 
 private function test_sequential_packs()
  test_sequential_packs=false
 
  call m_base.log_msg("[TEST RPL PACK SEQUENTIAL]")

  dim pv
  set pv=m_rpl_packs

  dim ip1,p1
  dim ip2,p2

  dim msg
  dim c
  
  c=0

  for ip1=0 to pv.count-1
   set p1=pv(ip1)

   for ip2=ip1+1 to pv.count-1
    set p2=pv(ip2)

    if(p1.terrain_global_id=p2.terrain_global_id)then
     if(p1.pack_id<p2.pack_id xor p1.pack_date_create<=p2.pack_date_create)then
      msg="Terrain:"&cstr(p1.terrain_name)&vbCrLf&_
          "Pack1:"&su_fnum(p1.pack_id,6)&" Date Create:"&su_date_to_str(p1.pack_date_create)&vbCrLf& _
          "Pack2:"&su_fnum(p2.pack_id,6)&" Date Create:"&su_date_to_str(p2.pack_date_create)

      call m_base.log_error(-1,msg)

      c=c+1
     end if
    end if 'equal terrains
   next 'ip2
  next 'ip1
  
  test_sequential_packs=(c=0)
 end function 'test_sequential_packs
end class 't_task_for_scan_rpl_packs
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
